#include "shift_register.h"
#include <avr\io.h>


#define DATA_OUT_PORT	PORTD
#define DATA_OUT_PIN	2
#define DDR_DATA_OUT	DDRD

#define DATA_IN_PORT	PIND
#define DATA_IN_PIN		5

#define CLK_PORT		PORTD
#define CLK_PIN			3
#define DDR_CLK			DDRD

#define OUTPUT_LATCH_PORT	PORTD
#define OUTPUT_LATCH_PIN	4
#define DDR_OUTPUT_LATCH	DDRD

#define INPUT_LATCH_PORT	PORTD
#define INPUT_LATCH_PIN		6
#define DDR_INPUT_LATCH		DDRD


/*********************************************************
*	name:	toggleClock
*	param: 	none
*	desc:	clocks the shift registers one times
*********************************************************/
void toggleClock(void)
{
	CLK_PORT |= (1<<CLK_PIN);
	CLK_PORT &= ~(1<<CLK_PIN);
}

/*********************************************************
*	name:	toggleOutputLatch
*	param: 	none
*	desc:	clocks the output latch one times
*********************************************************/
void toggleOutputLatch(void)
{
	OUTPUT_LATCH_PORT |= (1<<OUTPUT_LATCH_PIN);
	OUTPUT_LATCH_PORT &= ~(1<<OUTPUT_LATCH_PIN);
}

/*********************************************************
*	name:	toggleInputLatch
*	param: 	none
*	desc:	clocks the input latch one times
*********************************************************/
void toggleInputLatch(void)
{
	INPUT_LATCH_PORT |= (1<<INPUT_LATCH_PIN);
	INPUT_LATCH_PORT &= ~(1<<INPUT_LATCH_PIN);
}

/*********************************************************
*	name:	SetOutputPins
*	param: 	out - new pinstate
*	desc:	updates the state of the output pins
*********************************************************/
void ShiftRegister_SetOutput(uint16_t out)
{
    for(uint16_t n = 1; n > 0;n<<=1)
    {
		if(out & n)
			DATA_OUT_PORT |= (1<<DATA_OUT_PIN);
		else
			DATA_OUT_PORT &= ~(1<<DATA_OUT_PIN);
		toggleClock();
	}
	toggleOutputLatch();
}

/*********************************************************
*	name:	GetInputPins
*	param: 	none
*	desc:	returns the state of the input pins
*********************************************************/
uint16_t ShiftRegister_GetInput()
{
	uint16_t in = 0;

	toggleInputLatch();

	for(uint16_t n = 0x8000; n > 0;n>>=1)
	{
		if(DATA_IN_PORT & (1<<DATA_IN_PIN))
			in |= n;

		toggleClock();
	}

	return in;
}

/*********************************************************
*	name:	ShiftRegPinsInit
*	param: 	none
*	desc:	initializes shift registers
*********************************************************/
void ShiftRegister_Init(void)
{
	DDR_DATA_OUT |= (1<<DATA_OUT_PIN);
	DDR_CLK |= (1<<CLK_PIN);
	DDR_OUTPUT_LATCH |= (1<<OUTPUT_LATCH_PIN);
	DDR_INPUT_LATCH |= (1<<INPUT_LATCH_PIN);

	ShiftRegister_SetOutput(0);
}
