#include "ring_buffer8.h"

#include <avr/io.h>
#include <avr/interrupt.h>


#define     TX_BUFFER_SIZE      30
#define     RX_BUFFER_SIZE      30

static struct SRingBuffer8 tx_buffer;
static struct SRingBuffer8 rx_buffer;

static RingBuffer8Data     tx_data[TX_BUFFER_SIZE];
static RingBuffer8Data     rx_data[RX_BUFFER_SIZE];


ISR(USART_RXC_vect)
{
	uint8_t t = UDR;
	RingBuffer8_Push(t,&rx_buffer);
}

ISR(USART_UDRE_vect)
{
	if(RingBuffer8_Size(&tx_buffer))
		UDR = RingBuffer8_Pop(&tx_buffer);
	else
		UCSRB &= ~(1<<UDRIE);
}

void Uart_Init(void)
{
	RingBuffer8_Init(&tx_buffer,tx_data,TX_BUFFER_SIZE);
	RingBuffer8_Init(&rx_buffer,rx_data,RX_BUFFER_SIZE);

    //19200 Bit/s
	UBRRH = 0;
	UBRRL = 47;

	UCSRB |= (1<<RXEN) | (1<<TXEN) | (1<<RXCIE);
	//8 data, 1 stop bit, no parity
	UCSRC |= (1<<URSEL)|(3<<UCSZ0);
}

void	Uart_SendByte(uint8_t byte)
{
	RingBuffer8_Push(byte,&tx_buffer);
	UCSRB |= (1<<UDRIE);
}

uint8_t		Uart_GetByte(void)
{
	return RingBuffer8_Pop(&rx_buffer);
}

uint8_t		Uart_DataAvaible(uint8_t num)
{
	return RingBuffer8_Size(&rx_buffer) >= num ? 1 : 0;
}

uint8_t		Uart_TransferPossible(uint8_t num)
{
	RingBuffer8Size s = RingBuffer8_Size(&tx_buffer);
//if the buffer is completely full, head = tail -> size = 0. To prevent this error, leave one byte empty.
	if(s)
		return  s - 1 >= num ? 1 : 0;
	else
		return num < TX_BUFFER_SIZE - 1 ? 1 : 0;
}
